<?php
##########################################################################
# *******************  SX CONTENT MANAGEMENT SYSTEM  ******************* #
# *        Copyright © Alexander Voloshin * All Rights Reserved        * #
# ********************************************************************** #
# *  https://status-x.ru/  *  License GNU GPL v2  *  info@status-x.ru  * #
# ********************************************************************** #
##########################################################################
if (!defined('SX_DIR')) {
    header('Refresh: 0; url=/index.php?p=notfound', true, 404); exit;
}

class Keywords extends Magic {

    protected $_count = 0;       // Системный параметр - количество слов в массиве
    protected $_array = array(); // Системный параметр - массив слов
    protected $_options = array(// Массив параметров
        'stop'  => array(), // Дополнительный массив стоп-слов
        'check' => true, // Чистка массива по списку стоп-слов, true - активно, false - не очищать
        'count' => 25, // Количество слов на выходе
        'sep'   => ', ', // Разделитель выводимых фраз
        'len'   => 5, // Минимальная длина слова в методе формирующем по одному слова
        'lev'   => 2, // Минимальное количество вхождений одного слова в тексте
        'len2'  => 3, // Минимальная длина слова в методе формирующем по два слова
        'lev2'  => 3, // Минимальное количество вхождений фразы их двух слов в тексте
        'len3'  => 3, // Минимальная длина слова в методе формирующем по три слова
        'lev3'  => 2, // Минимальное количество вхождений фразы их трех слов в тексте
    );

    /* Метод создания часто встречаемых фраз в тексте */
    public function create($content, $options = array()) {
        $count = $this->options($options);
        $content = $this->add($content);
        $result = array(
            $this->phrase($count, 3),
            $this->phrase($count, 2),
            $this->words($count)
        );
        return $this->text($result);
    }

    /* Метод добавления массива слов или проста текста */
    public function add($content) {
        if (!empty($content)) {
            if (!is_array($content)) {
                $content = $this->clean($content);
                $content = explode(' ', $content);
            }
            $content = array_map('trim', $content);
            $content = $this->stop($content);
            $this->_array = array_merge($this->_array, $content);
            $this->_count = count($this->_array);
        }
    }

    /* Метод добавления параметров и установки умолчаний */
    protected function options($options = array()) {
        $this->_options = (array) $options + $this->_options;
        return $this->_options['count'];
    }

    /* Метод подсчета количества фраз для вывода */
    protected function phrase(&$count, $num) {
        $method = 'words' . $num;
        $part = round($count / $num);
        $part = round($part / $num);
        $array = $this->$method($part);
        $count -= count($array) * $num;
        return $array;
    }

    /* Метод склеивания фраз в одну строку  */
    protected function text($array) {
        $result = array();
        if (!empty($array)) {
            $array = array_reverse($array);
            foreach ($array as $value) {
                if (!empty($value)) {
                    $result[] = implode($this->_options['sep'], $value);
                }
            }
        }
        return implode($this->_options['sep'], $result);
    }

    /* Метод очистки контента */
    protected function clean($content) {
        $content = $this->_text->lower($content);
        $content = strip_tags($content);
        $content = str_replace(array('&amp;amp;amp;', '&amp;amp;', '&amp;'), '&', $content);
        $content = preg_replace(array('/&[#a-z\d]{2,6};/u', '/[^\w- ]/iu', '/\s+/u'), ' ', $content);
        return trim($content);
    }

    /* Метод удаления стоп-слов из массива слов */
    protected function stop($array) {
        if ($this->_options['check'] === true) {
            $ru = array('алло', 'бежал', 'без', 'белый', 'близко', 'более', 'больше', 'большой', 'боюсь', 'будем', 'будет', 'будете', 'будешь', 'будто', 'буду', 'будут', 'будь', 'бывает', 'бывь', 'был', 'была', 'были', 'было', 'было', 'бытие', 'быть', 'важная', 'важное', 'важные', 'важный', 'вам', 'вами', 'вас', 'ваш', 'ваша', 'ваше', 'ваши', 'вверх', 'вдали', 'вдоль', 'вдруг', 'ведь', 'везде', 'великий', 'вероятно', 'весь', 'взял', 'взять', 'видел', 'видеть', 'визит', 'влево', 'вместе', 'вместо', 'внезапно', 'вниз', 'вниз', 'внизу', 'внутри', 'возможно', 'возраст', 'вокруг', 'воля', 'вон', 'восемнадцатый', 'восемнадцать', 'восемь', 'восьмой', 'вот', 'впрочем', 'времени', 'время', 'всё', 'все', 'всегда', 'всего', 'всей', 'всем', 'всеми', 'всему', 'всех', 'всею', 'встретились', 'встроенные', 'всю', 'всюду', 'вся', 'второй', 'выбрать', 'вызов', 'выключено', 'выполнить', 'вырезать', 'вырос', 'высокая', 'высокий', 'выше', 'где', 'главная', 'глагол', 'глубокий', 'гнев', 'говорил', 'говорит', 'говорить', 'год', 'года', 'году', 'горе', 'горячий', 'готово', 'давай', 'давать', 'давно', 'дает', 'даже', 'далее', 'далеко', 'дальше', 'даром', 'два', 'двадцатый', 'двадцать', 'две', 'двенадцатый', 'двенадцать', 'двигаться', 'двух', 'девятнадцатый', 'девятнадцать', 'девятый', 'девять', 'действительно', 'дел', 'делать', 'день', 'держал', 'держать', 'десятков', 'десятый', 'десять', 'длинный', 'для', 'добавить', 'довольно', 'долго', 'должен', 'должно', 'достаточно', 'достижения', 'другая', 'другие', 'других', 'друго', 'другое', 'другой', 'его', 'ему', 'если', 'есть', 'ещё', 'еще', 'железо', 'животное', 'жизнь', 'забыл', 'завтра', 'закрыть', 'залив', 'занят', 'занята', 'занято', 'заняты', 'заполнить', 'затем', 'зато', 'зачем', 'зачем', 'здесь', 'зеленые', 'знак', 'знал', 'знать', 'значит', 'значительный', 'идет', 'из-за', 'известным', 'извините', 'или', 'имеет', 'именно', 'иметь', 'иметь', 'ими', 'имя', 'иначе', 'иногда', 'использование', 'каждая', 'каждое', 'каждые', 'каждый', 'кажется', 'как', 'как-то', 'какая', 'какой', 'кем', 'когда', 'кого', 'количество', 'ком', 'кому', 'конец', 'конечно', 'копия', 'которая', 'которого', 'которой', 'которые', 'который', 'которых', 'крик', 'кроме', 'кругом', 'кто', 'куда', 'купить', 'лет', 'либо', 'лишь', 'луна', 'лучше', 'лучший', 'любой', 'люди', 'мало', 'мальчик', 'между', 'меля', 'менее', 'меньше', 'меня', 'мере', 'мешок', 'миллионов', 'милый', 'миля', 'мимо', 'мира', 'мне', 'многие', 'много', 'многочисленная', 'многочисленное', 'многочисленные', 'многочисленный', 'множество', 'мной', 'мною', 'моё', 'мог', 'могут', 'мож', 'может', 'можно', 'можхо', 'мои', 'мой', 'молодые', 'мор', 'мочь', 'моя', 'наверху', 'навсегда', 'над', 'надо', 'назад', 'наиболее', 'найти', 'наконец', 'нам', 'нами', 'нас', 'насладиться', 'научили', 'начал', 'начала', 'начать', 'наш', 'наша', 'наше', 'наши', 'неё', 'него', 'недавно', 'недалеко', 'нее', 'ней', 'некоторые', 'некто', 'нельзя', 'нем', 'немного', 'нему', 'необходимым', 'непрерывно', 'нередко', 'несколько', 'нет', 'нею', 'нибудь', 'ниже', 'низко', 'никогда', 'никто', 'никуда', 'ними', 'них', 'ничего', 'ничто', 'ноги', 'ноль', 'номер', 'нужно', 'оба', 'объяснить', 'обычно', 'обычный', 'один', 'одиннадцатый', 'одиннадцать', 'одинокий', 'однажды', 'однако', 'одного', 'одной', 'одолжить', 'ожидать', 'окно', 'около', 'она', 'они', 'оно', 'опять', 'оружие', 'особенно', 'отредактировано', 'ответ', 'ответить', 'отовсюду', 'отсюда', 'оттуда', 'отчего', 'очень', 'падение', 'палка', 'партия', 'первый', 'перед', 'перерыв', 'печать', 'плохо', 'повесить', 'поворот', 'под', 'подготовка', 'поднять', 'подробнее', 'подходит', 'пожалуйста', 'поздно', 'позже', 'пока', 'покупать', 'половина', 'получил', 'получить', 'помощь', 'понимать', 'попробовать', 'попытка', 'пор', 'пора', 'порвал', 'поскольку', 'после', 'посреди', 'потом', 'потому', 'почему', 'почему', 'почти', 'пошел', 'поэтому', 'появиться', 'пребывание', 'предполагать', 'прекрасно', 'при', 'привели', 'привет', 'принадлежать', 'принес', 'приносить', 'принято', 'приходят', 'причина', 'пришел', 'про', 'провал', 'продается', 'продать', 'продолжить', 'произойдет', 'просто', 'против', 'процентов', 'пусть', 'путь', 'пытается', 'пытался', 'пятнадцатый', 'пятнадцать', 'пятый', 'пять', 'раз', 'разве', 'различны', 'разные', 'ранее', 'рано', 'раньше', 'рассмотреть', 'расти', 'решить', 'родился', 'рука', 'рядом', 'сам', 'сама', 'сами', 'самим', 'самими', 'самих', 'само', 'самого', 'самой', 'самом', 'самому', 'саму', 'самый', 'свое', 'своего', 'своей', 'свои', 'своих', 'свой', 'свою', 'сделал', 'сделка', 'себе', 'себя', 'сегодня', 'седьмой', 'сейчас', 'сейчас', 'семнадцатый', 'семнадцать', 'семь', 'сердится', 'серый', 'сих', 'сказал', 'сказал', 'сказала', 'сказать', 'сказать', 'сколько', 'скоро', 'следить', 'слишком', 'слышали', 'смотреть', 'сначала', 'снизу', 'снова', 'собака', 'собой', 'собою', 'совсем', 'согласен', 'содержать', 'сомнение', 'состоялась', 'состоянии', 'спасибо', 'спросить', 'средняя', 'стал', 'стать', 'сторона', 'стояли', 'страх', 'суть', 'схожи', 'съесть', 'так', 'такая', 'также', 'такие', 'такое', 'такой', 'там', 'твоё', 'твой', 'твоя', 'тебе', 'тебя', 'тело', 'тем', 'теми', 'темный', 'теперь', 'тетя', 'тех', 'тобой', 'тобою', 'тогда', 'того', 'тоже', 'той', 'только', 'том', 'тому', 'тот', 'точно', 'тою', 'требуются', 'третий', 'три', 'тринадцатый', 'тринадцать', 'туда', 'тут', 'ты', 'тысяч', 'тысячи', 'тянуть', 'угадать', 'уже', 'умереть', 'уметь', 'упал', 'усилие', 'услышать', 'ушел', 'хорошо', 'хотеть', 'хоть', 'хотя', 'хочешь', 'часто', 'чаще', 'чего', 'чего-то', 'человек', 'чем', 'чему', 'через', 'четвертый', 'четыре', 'четырнадцатый', 'четырнадцать', 'что', 'чтоб', 'чтобы', 'чувствовать', 'чуть', 'шестнадцатый', 'шестнадцать', 'шестой', 'шесть', 'шляпа', 'эта', 'эти', 'этим', 'этими', 'этих', 'это', 'этого', 'этой', 'этом', 'этому', 'этот', 'эту', 'являются');
            $en = array('able', 'about', 'above', 'act', 'add', 'afraid', 'after', 'again', 'against', 'age', 'ago', 'agree', 'all', 'almost', 'alone', 'along', 'already', 'also', 'although', 'always', 'am', 'amount', 'an', 'and', 'anger', 'angry', 'animal', 'another', 'answer', 'any', 'appear', 'apple', 'are', 'arrive', 'arm', 'arms', 'around', 'arrive', 'as', 'ask', 'at', 'attempt', 'aunt', 'away', 'back', 'bad', 'bag', 'bay', 'be', 'became', 'because', 'become', 'been', 'before', 'began', 'begin', 'behind', 'being', 'bell', 'belong', 'below', 'besidev', 'bestv', 'better', 'between', 'beyond', 'big', 'body', 'bone', 'born', 'borrow', 'both', 'bottom', 'box', 'boy', 'break', 'bring', 'brought', 'bug', 'built', 'busy', 'but', 'buy', 'by', 'call', 'came', 'can', 'cause', 'choose', 'close', 'close', 'consider', 'come', 'consider', 'considerable', 'contain', 'continue', 'could', 'cry', 'cut', 'dare', 'dark', 'deal', 'vdear', 'decide', 'deep', 'did', 'die', 'do', 'does', 'dog', 'done', 'doubt', 'down', 'during', 'each', 'ear', 'early', 'eat', 'effort', 'either', 'else', 'end', 'enjoy', 'enough', 'enter', 'even', 'ever', 'every', 'except', 'expect', 'explain', 'fail', 'fall', 'far', 'fat', 'favor', 'fear', 'feel', 'feet', 'fell', 'felt', 'few', 'fill', 'find', 'fit', 'fly', 'follow', 'for', 'forever', 'forget', 'from', 'front', 'gave', 'get', 'gives', 'goes', 'gone', 'good', 'got', 'gray', 'great', 'green', 'grew', 'grow', 'guess', 'had', 'half', 'hang', 'happen', 'has', 'hat', 'have', 'he', 'hear', 'heard', 'held', 'hello', 'help', 'her', 'here', 'hers', 'high', 'hill', 'him', 'his', 'hit', 'hold', 'hot', 'how', 'however', 'in', 'indeed', 'instead', 'into', 'iron', 'is', 'it', 'its', 'just', 'keep', 'kept', 'knew', 'know', 'known', 'late', 'least', 'led', 'left', 'lend', 'less', 'let', 'like', 'likely', 'likr', 'lone', 'long', 'look', 'lot', 'make', 'many', 'may', 'me', 'mean', 'met', 'might', 'mile', 'mine', 'moon', 'more', 'most', 'move', 'much', 'must', 'my', 'near', 'nearly', 'necessary', 'neither', 'never', 'next', 'no', 'none', 'nor', 'not', 'note', 'nothing', 'now', 'number', 'of', 'off', 'often', 'oh', 'on', 'once', 'only', 'or', 'other', 'ought', 'our', 'out', 'please', 'prepare', 'probable', 'pull', 'pure', 'push', 'put', 'raise', 'ran', 'rather', 'reach', 'realize', 'reply', 'require', 'rest', 'run', 'said', 'same', 'sat', 'saw', 'say', 'see', 'seem', 'seen', 'self', 'sell', 'sent', 'separate', 'set', 'shall', 'she', 'should', 'side', 'sign', 'since', 'so', 'sold', 'some', 'soon', 'sorry', 'stay', 'step', 'stick', 'still', 'stood', 'such', 'sudden', 'suppose', 'take', 'taken', 'talk', 'tall', 'tell', 'ten', 'than', 'thank', 'that', 'the', 'their', 'them', 'then', 'there', 'therefore', 'these', 'they', 'this', 'those', 'though', 'through', 'till', 'to', 'today', 'told', 'tomorrow', 'too', 'took', 'tore', 'tought', 'toward', 'tried', 'tries', 'trust', 'try', 'turn', 'two', 'under', 'until', 'up', 'upon', 'us', 'use', 'usual', 'various', 'verb', 'very', 'visit', 'want', 'was', 'we', 'well', 'went', 'were', 'what', 'when', 'where', 'whether', 'which', 'while', 'white', 'who', 'whom', 'whose', 'why', 'will', 'with', 'within', 'without', 'would', 'yes', 'yet', 'you', 'young', 'your', 'br', 'img', 'p', 'lt', 'gt', 'quot', 'copy');
            $array = array_diff($array, $ru, $en, $this->_options['stop']);
        }
        return $array;
    }

    /* Метод формирует фразы из двух слов */
    protected function words($count = 5) {
        $len = $this->_options['len'];
        $result = array();
        foreach ($this->_array as $value) {
            if (isset($value{$len}) && !is_numeric($value)) {
                $result[$value] = !isset($result[$value]) ? 1 : $result[$value] + 1;
            }
        }
        return $this->result($result, $count, $this->_options['lev']);
    }

    /* Метод формирует список слов по частоте вхождений */
    protected function words2($count = 5) {
        $len = $this->_options['len2'];
        $x = $this->_array;
        $result = array();
        for ($i = 0; $i < $this->_count - 1; $i++) {
            if (isset($x[$i]{$len}, $x[$i + 1]{$len}) && $x[$i] != $x[$i + 1]) {
                $key = $x[$i] . ' ' . $x[$i + 1];
                $result[$key] = !isset($result[$key]) ? 1 : $result[$key] + 1;
            }
        }
        return $this->result($result, $count, $this->_options['lev2']);
    }

    /* Метод формирует фразы из трех слов */
    protected function words3($count = 5) {
        $len = $this->_options['len3'];
        $x = $this->_array;
        $result = array();
        for ($i = 0; $i < $this->_count - 2; $i++) {
            if (isset($x[$i]{$len}, $x[$i + 1]{$len}, $x[$i + 2]{$len}) && $x[$i] != $x[$i + 1] && $x[$i + 1] != $x[$i + 2]) {
                $key = $x[$i] . ' ' . $x[$i + 1] . ' ' . $x[$i + 2];
                $result[$key] = !isset($result[$key]) ? 1 : $result[$key] + 1;
            }
        }
        return $this->result($result, $count, $this->_options['lev3']);
    }

    /* Метод подсчета вхождений фраз и обрезка по количеству  */
    protected function result($array, $count = 5, $min = 2) {
        $result = array();
        if (!empty($array)) {
            arsort($array);
            $array = array_diff($array, range(1, $min - 1));
            if (!empty($array)) {
                $array = array_keys($array);
                $result = array_slice($array, 0, $count);
            }
        }
        return $result;
    }

}
